/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef DAQPARAMCONTROLLER_H
#define DAQPARAMCONTROLLER_H

#include <Controller.h>
#include "drivers/stereo_elec/StereoElecDriver.h"
#include "drivers/caen_syx527/CAENSYx527Driver.h"
#include "controllers/common/acquisition/mode/TimeAcquisitionController.h"

namespace stereo_daq {

static const char TrigAmp[4] = "Amp";
static const char TrigSum[4] = "Sum";

/*!
 * \brief Commands select to be used as a thread.
 */
class Select: private controller::Command {

public:
	Select(ExperimentController * c) :
			controller::Command(c, "select", &Select::select, this) {
	}

	virtual void select() = 0;
	void selectCommand(bool logging = false) {
		command(logging, true);
	}
};

/*!
 * \brief Commands set to be used as a thread.
 */
class Set: private controller::Command {

public:
	Set(ExperimentController * c) :
			controller::Command(c, "set", &Set::set, this) {
	}

	virtual void set() = 0;
	void setCommand(bool logging = false) {
		command(logging, true);
	}
};

/*!
 * \class DAQParamController
 */

class DAQParamController: public acquisition::TimeAcquisitionController, public Select, public Set {

public:

	//! Type of controller
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 *
	 * It is the default constructor called by the creator class ExperimentControllerCreator.
	 *
	 * \param[in] name the name of the controller
	 */
	DAQParamController(const std::string& name);

	/*!
	 * \brief Copy constructor
	 *
	 * \param[in] controller the controller to copy
	 */
	DAQParamController(const DAQParamController& controller);

	/*!
	 * \brief Destructor
	 */
	virtual ~DAQParamController();

	/*!
	 * \brief Method called before changing the property value
	 *
	 * This method is called after setting configuration during the creation of controller.
	 */
	virtual void postConfiguration();

	/*!
	 * Properties
	 */
	Property<int32> daqmode; // 0 : normal, 1: debug
	Property<int32> trigmode; //
	Property<int32> reqcoinc;
	Property<int32> nsample;
	Property<string> fileformat;
	Property<double> filtercoef;

	Property<int32> nfeboard;
	// 1 value for each fe board
	DynamicProperty<int32> ntot;
	DynamicProperty<int32> ntail;
	DynamicProperty<int32> pretrig;
	DynamicProperty<int32> cfddelay;
	DynamicProperty<int32> cfdfraction;
	DynamicProperty<int32> precfd;
	DynamicProperty<bool> trigchannel;
	DynamicProperty<bool> trigsum03;
	DynamicProperty<bool> trigsum47;
	DynamicProperty<bool> trigsum07;
	DynamicProperty<string> trigquantity;  // Charge or amplitude
	DynamicProperty<int32> slotindex;
	DynamicProperty<int32> sumthres;
	DynamicProperty<int32> nsum;

	Property<int32> ntotchannel;
	// 8 values for each fe board
	DynamicProperty<int32> itotchannel;
	DynamicProperty<int32> islot;
	DynamicProperty<int32> ichannel;
	DynamicProperty<std::string> namechannel;
	DynamicProperty<bool> gain20;
	DynamicProperty<int32> trigthres;
	DynamicProperty<int32> cfdthres;
	DynamicProperty<bool> isselected;

	Property<string> selectchoice;
	Property<int32> selectslot;
	Property<string> setparam;
	Property<string> setvalue;

protected:

	DriverPtr<stereo_elec::StereoElecDriver> m_ELECDriver; //! The driver linked to the controller
	DriverPtr<caen_syx527::CAENSYx527Driver> m_HVDriver; //! The HV Caen driver

	/*!
	 * \brief select command
	 */
	virtual void select();

	/*!
	 * \brief select command
	 */
	virtual void set();

	/*!
	 * \brief start
	 *
	 */
	virtual void start() {
	}

	/*!
	 * \brief start
	 *
	 */
	virtual void pause() {
	}

	/*!
	 * \brief Stop command
	 */
	virtual void resume() {
	}

	/*!
	 * \brief synchronize read command
	 */
	virtual void synchronizeRead();

	/*!
	 * \brief Write parameters on the drivers
	 */
	virtual void writeParams();

	/*!
	 * \brief Update parameters from the drivers
	 */
	virtual void updateParams() {

	}

	/*!
	 * \brief Resize the detector for z (tof) and w (slice) coordinates
	 * \throws CannotOpenFile Cannot open the file
	 * \throws SlicesFileError Slice file syntax error
	 * \throws TooManySlices Too many slices
	 * \throws SliceTooBig a slice is too big (time)
	 */
	virtual void resize() throw(CannotOpenFile, SlicesFileError, TooManySlices, SliceTooBig);

	/*!
	 * \brief Check if the acquisition using energy mode
	 * \return Energy mode used?
	 */
	virtual bool useEnergy() const {
		return false;
	}

private:

	/*
	 * local values
	 */
	bool m_StartActivated; //! Started activated flag

	void refreshDAQModeProperty(int32 value) throw (CannotSetValue);
	void refreshTrigModeProperty(int32 value) throw (CannotSetValue);
	void refreshReqCoincProperty(int32 value) throw (CannotSetValue);
	void refreshNSampleProperty(int32 value) throw (CannotSetValue);
	void refreshFileFormatProperty(const std::string& value) throw (CannotSetValue);
	void refreshFilterCoefProperty(double value) throw (CannotSetValue);
	void refreshNFEBoardProperty(int32 value) throw (CannotSetValue);
	void refreshNTotProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshNTailProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshPretrigProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshTrigQuantityProperty(int32 index, const std::string& value) throw (CannotSetValue);
	void refreshTrigChannelProperty(int32 index, bool value) throw (CannotSetValue);
	void refreshTrigSum03Property(int32 index, bool value) throw (CannotSetValue);
	void refreshTrigSum47Property(int32 index, bool value) throw (CannotSetValue);
	void refreshTrigSum07Property(int32 index, bool value) throw (CannotSetValue);
	void refreshCFDDelayProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshCFDFractionProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshPreCFDProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshSumThresProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshNSumProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshGain20Property(int32 index, bool value) throw (CannotSetValue);
	void refreshTrigThresProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshCFDThresProperty(int32 index, int32 value) throw (CannotSetValue);
	void refreshNameChannelProperty(int32 index, const std::string& value) throw (CannotSetValue);

	void updateTriggerIPAddress();
	void updateNFEBoard();
	void updateBoardIPAddress(int32 index);
	void updateNameChannel0(int32 index);
	void updateNameChannel1(int32 index);
	void updateNameChannel2(int32 index);
	void updateNameChannel3(int32 index);
	void updateNameChannel4(int32 index);
	void updateNameChannel5(int32 index);
	void updateNameChannel6(int32 index);
	void updateNameChannel7(int32 index);

};
}
#endif // DAQPARAMCONTROLLER_H
